<?php
/**
 * @file
 * ImageField module integration.
 */

/**
 * Implements hook_widget_settings_alter().
 */
function fbphotosync_widget_settings_alter(&$settings, $op, $widget) {
  if (function_exists($function = "fbphotosync_widget_settings_alter_{$op}")) {
    $function($settings, $widget);
  }
}

/**
 * Form callback for widget settings form.
 */
function fbphotosync_widget_settings_alter_form(&$settings, $widget) {
  if ($widget['module'] == 'imagefield') {
    $facebook = fbphotosync_facebook_sdk();
    ctools_include('dependent');

    // Settings form.
    $settings['fbphotosync'] = array(
      '#type' => 'fieldset',
      '#title' => t('Facebook Photo Sync'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#weight' => isset($settings['default']) ? $settings['default']['#weight'] + 1: 11,
      '#tree' => TRUE,
    );
    $settings['fbphotosync']['status'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use Facebook Photo Sync'),
      '#default_value' => isset($widget['fbphotosync']['status']) ? $widget['fbphotosync']['status'] : FALSE,
    );

    // Account settings.
    $settings['fbphotosync']['account'] = array(
      '#type' => 'fieldset',
      '#title' => t('Account settings'),
      '#process' => array('ctools_dependent_process'),
      '#dependency' => array('edit-fbphotosync-status' => array(TRUE)),
      '#input' => TRUE,
      '#id' => 'edit-fbphotosync-account',
      '#prefix' => '<div id="edit-fbphotosync-account-wrapper">',
      '#suffix' => '</div>',
    );
    $settings['fbphotosync']['account']['id'] = array(
      '#type' => 'select',
      '#title' => t('Account'),
      '#options' => fbphotosync_account_ids($facebook),
      '#default_value' => isset($widget['fbphotosync']['account']['id']) ? $widget['fbphotosync']['account']['id'] : key(fbphotosync_account_ids($facebook)),
      '#required' => TRUE,
      '#process' => array('ctools_dependent_process'),
      '#dependency' => array('edit-fbphotosync-status' => array(TRUE)),
    );

    // Album settings.
    $settings['fbphotosync']['album'] = array(
      '#type' => 'fieldset',
      '#title' => t('Album settings'),
      '#process' => array('ctools_dependent_process'),
      '#dependency' => array('edit-fbphotosync-status' => array(TRUE)),
      '#input' => TRUE,
      '#id' => 'edit-fbphotosync-album',
      '#prefix' => '<div id="edit-fbphotosync-album-wrapper">',
      '#suffix' => '</div>',
    );
    $settings['fbphotosync']['album']['type'] = array(
      '#type' => 'select',
      '#title' => t('Type'),
      '#options' => array(
        // 'existing' => t('Existing album'),
        'dynamic' => t('Dynamically generated album'),
      ),
      '#default_value' => isset($widget['fbphotosync']['album']['type']) ? $widget['fbphotosync']['album']['type'] : 0,
      '#required' => TRUE,
    );
    // $settings['fbphotosync']['album']['existing'] = array(
    //   '#type' => 'select',
    //   '#title' => t('Album'),
    //   '#options' => array(),
    //   '#default_value' => isset($widget['fbphotosync']['album']['existing']) ? $widget['fbphotosync']['album']['existing'] : 0,
    //   '#process' => array('ctools_dependent_process'),
    //   '#dependency' => array(
    //     'edit-fbphotosync-album-type' => array('existing'),
    //   ),
    // );
    $settings['fbphotosync']['album']['dynamic'] = array();
    $settings['fbphotosync']['album']['dynamic']['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#default_value' => isset($widget['fbphotosync']['album']['dynamic']['name']) ? $widget['fbphotosync']['album']['dynamic']['name'] : '',
      '#element_validate' => array('_fbphotosync_imagefield_element_validate'),
      '#process' => array('ctools_dependent_process'),
      '#dependency' => array(
        'edit-fbphotosync-album-type' => array('dynamic'),
      ),
    );
    $settings['fbphotosync']['album']['dynamic']['message'] = array(
      '#type' => 'textarea',
      '#title' => t('Description'),
      '#default_value' => isset($widget['fbphotosync']['album']['dynamic']['message']) ? $widget['fbphotosync']['album']['dynamic']['message'] : '',
      '#process' => array('ctools_dependent_process'),
      '#dependency' => array(
        'edit-fbphotosync-album-type' => array('dynamic'),
      ),
    );

    // Image settings.
    $settings['fbphotosync']['image'] = array(
      '#type' => 'fieldset',
      '#title' => t('Image settings'),
      '#process' => array('ctools_dependent_process'),
      '#dependency' => array('edit-fbphotosync-status' => array(TRUE)),
      '#input' => TRUE,
      '#id' => 'edit-fbphotosync-image',
      '#prefix' => '<div id="edit-fbphotosync-image-wrapper">',
      '#suffix' => '</div>',
    );
    $settings['fbphotosync']['image']['cardinality'] = array(
      '#type' => 'select',
      '#title' => t('Number of values'),
      '#options' => array(0 => t('Unlimited')) + drupal_map_assoc(range(1, 10)),
      '#default_value' => isset($widget['fbphotosync']['image']['cardinality']) ? $widget['fbphotosync']['image']['cardinality'] : 0,
      '#required' => TRUE,
      '#description' => t('Maximum number of values of images to be uploaded to Facebook.'),
    );
    $settings['fbphotosync']['image']['message'] = array(
      '#type' => 'textarea',
      '#title' => t('Description'),
      '#default_value' => isset($widget['fbphotosync']['image']['message']) ? $widget['fbphotosync']['image']['message'] : '',
    );

    // Replacement patterns.
    $settings['fbphotosync']['tokens'] = array(
      '#type' => 'fieldset',
      '#title' => t('Replacement patterns'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#description' => theme('token_tree', array('node', 'field', 'global')),
      '#process' => array('ctools_dependent_process'),
      '#dependency' => array('edit-fbphotosync-status' => array(TRUE)),
      '#input' => TRUE,
      '#id' => 'edit-fbphotosync-tokens',
      '#prefix' => '<div id="edit-fbphotosync-tokens-wrapper">',
      '#suffix' => '</div>',
    );
  }
}

/**
 * Element validation callback for Dynamic album fields.
 */
function _fbphotosync_imagefield_element_validate($element, &$form_state) {
  if ($form_state['values']['fbphotosync']['status'] == TRUE && $form_state['values']['fbphotosync']['album']['type'] == 'dynamic' && empty($element['#value'])) {
    form_error($element, t('@field field is required.', array('@field' => $element['#title'])));
  }
}

/**
 * Save callback for widget settings form.
 */
function fbphotosync_widget_settings_alter_save(&$settings, $widget) {
  if ($widget['widget_module'] == 'imagefield') {
    $settings[] = 'fbphotosync';
  }
}

/**
 * Implements hook_nodeapi().
 */
function fbphotosync_nodeapi(&$node, $op, $a3 = NULL, $a4 = NULL) {
  switch ($op) {
    case 'insert':
      $settings = variable_get('fbphotosync_settings', array());
      $content_type = content_types($node->type);
      foreach ($content_type['fields'] as $instance) {
        if ($instance['widget']['module'] == 'imagefield' && isset($instance['widget']['fbphotosync']) && $instance['widget']['fbphotosync']['status'] == TRUE && isset($node->{$instance['field_name']})) {
          if ($facebook = fbphotosync_facebook_sdk()) {
            if (fbphotosync_facebook_sdk_is_authenticated($facebook, $settings['access_token'])) {
              // Set file upload support.
              $facebook->setFileUploadSupport(TRUE);

              // Extract account type and ID from instance settings.
              list($account_type, $account_id) = explode(':', $instance['widget']['fbphotosync']['account']['id']);

              // Process field files.
              foreach ($node->{$instance['field_name']} as $delta => $field) {
                if ($instance['widget']['fbphotosync']['image']['cardinality'] == 0 || $instance['widget']['fbphotosync']['image']['cardinality'] > $delta) {
                  $token_data = array(
                    'node' => $node,
                    'field' => $field,
                    'global' => NULL,
                  );

                  // Account type specific logic.
                  switch ($account_type) {
                    case 'application':
                    case 'page':
                      $response = $facebook->api("/{$account_id}?fields=access_token");
                      $access_token = isset($response['access_token']) ? $response['access_token'] : NULL;
                  }

                  // Album type specific logic.
                  switch ($instance['widget']['fbphotosync']['album']['type']) {
                    case 'dynamic':
                      // Process album name and description.
                      $args = array(
                        'name' => $instance['widget']['fbphotosync']['album']['dynamic']['name'],
                        'message' => $instance['widget']['fbphotosync']['album']['dynamic']['message'],
                      );
                      foreach ($token_data as $token_type => $object) {
                        $args['name'] = token_replace($args['name'], $token_type, $object);
                        $args['message'] = token_replace($args['message'], $token_type, $object);
                      }

                      // Check if album already exists.
                      $albums = $facebook->api("/{$account_id}/albums");
                      $album_exists = FALSE;
                      foreach ($albums['data'] as $album) {
                        if ($album['name'] == $args['name']) {
                          $album_exists = TRUE;
                          break;
                        }
                      }

                      // Create new album.
                      if (!$album_exists) {
                        // Add access_token if present.
                        if (isset($access_token)) {
                          $args['access_token'] = $access_token;
                        }

                        $album = $facebook->api("/{$account_id}/albums", 'post', $args);
                      }
                      break;

                    // case 'existing':
                  }

                  // Facebook expects the realpath to the file prepended with a '@'.
                  $args = array(
                    'image' => '@' . realpath($field['filepath']),
                  );
                  // Add access_token if present.
                  if (isset($access_token)) {
                    $args['access_token'] = $access_token;
                  }

                  // Process image message/description.
                  $args['message'] = $instance['widget']['fbphotosync']['image']['message'];
                  foreach ($token_data as $token_type => $object) {
                    $args['message'] = token_replace($args['message'], $token_type, $object);
                  }

                  // Upload the image file.
                  // @TODO - Store the Facebook Photo ID aginst the File ID.
                  $data = $facebook->api("/{$album['id']}/photos", 'post', $args);
                }
              }
            }
          }
        }
      }
      break;
  }
}
